/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.reqcoverage.eclipse.wizards;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.unitesk.requality.eclipse.wizards.ITestGenWizard;

public class FileContentPage extends WizardPage {
   private Text containerText, containerViewText;
   private ModifyListener ml;
   private DialogSettings store;
   /**
    * Constructor for SampleNewWizardPage.
    * 
    * @param pageName
    */
 
   public ITestGenWizard getTestWizard(){
	   return (ITestGenWizard)getWizard();
   }
   public FileContentPage( DialogSettings settings) {
      super( "fileContentsPage");
      setPageComplete(false);
      setDescription( "Please verify file contents." );
      this.store = settings;
   }
   private String getQid(){
	   String qid = storedtargetQiD;
	   if(qid == null)
		   qid = ((ITestGenWizard) getWizard()).getTargetQID();
	   if(qid == null)
		   qid = getDefaultQID();
	   return qid;
   }
   
   /**
    * @see IDialogPage#createControl(Composite)
    */
   public void createControl( Composite parent ) {
      Composite container= new Composite( parent, SWT.NULL );
      GridLayout layout= new GridLayout();
      layout.numColumns= 1;
      layout.verticalSpacing= 9;
      container.setLayout( layout );
      Composite fileSel = new Composite(container, SWT.NONE);
      fileSel.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
      fileSel.setLayout(new GridLayout(2,false)); 
      Label label= new Label( container, SWT.NULL );
      label.setText( "&Comment:" );
      
      containerText= new Text( container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL| SWT.H_SCROLL);
      
      GridData gd= new GridData( GridData.FILL_BOTH );
      gd.minimumHeight=200;
      containerText.setLayoutData( gd );
      ml = new ModifyListener() {
         public void modifyText( ModifyEvent e ) {
        	 if(store.get("commentText")==null || !containerText.getText().equals(store.get("commentText"))){
        		 dialogChanged();
        		 store.put("commentText", containerText.getText());
        		 updateState();
        	 }
         }
      };
      containerText.addModifyListener( ml );
      containerText.setFocus();
      

      label = new Label( container, SWT.NULL );
      label.setText( "&Preview:" );
      
      containerViewText = new Text( container, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL| SWT.H_SCROLL);
      containerViewText.setEditable(false);
      gd = new GridData( GridData.FILL_BOTH );
      containerViewText.setLayoutData( gd );
      
      setTargetQID(getQid(), true);
      
      updateState();
      setControl( container );
      dialogChanged();
   }
   private String getDefaultQID() {
	   return "Element Qualifying Id";
   }

/**
    * Ensures that both text fields are set.
    */
   private void dialogChanged() {
	   //TODO check
      updateStatus( null );
      String qid = ((ITestGenWizard)getWizard()).getTargetQID();
      if(qid == null)
    	  qid = getDefaultQID();
   }
   
   private void updateStatus( String message ) {
      setErrorMessage( message );
      setPageComplete( message == null );
   }
   
   public String getTSDescription() {
      return containerViewText.getText();
   }
   private String storedtargetQiD;
   
   public void setTargetQID(String targetqid, boolean local) {
	   if(targetqid == null)
		   targetqid = getDefaultQID();
	   storedtargetQiD = targetqid;
	   if(containerText != null && containerViewText != null)
		   containerViewText.setText(getNewFileContents());
   }
   public void updateState() {
	   String text = getComment();
	   if(containerText!=null && !containerText.getText().equals(text))
		   containerText.setText(text);
	   if(containerViewText!=null)
		   containerViewText.setText(getNewFileContents());
	   if(store.get("filePath")==null || store.get("filePath").equals(""))
		   updateStatus("File is not selected!");
	   else
		   if(store.get("commentText")==null)
			   updateStatus("Comment text is not selected!");
		   else
			   updateStatus(null);
   }
   private String getComment() {
	   String text = store.get("commentText");
	   if(text == null)
		   text = "//@requirement ${nodeQID}";
	   return text;
   }
   private String getContent() {
	   String s = store.get("filePath");
	   if(s==null || s.equals(""))
		   return "";
	   IFile f = ResourcesPlugin.getWorkspace().getRoot().getFile(Path.fromOSString(s));
	   StringBuffer sb = new StringBuffer();
	   if(f.exists()&&f.isAccessible()){
		   FileReader fr;
		   try {
			   InputStreamReader isr = new InputStreamReader(f.getContents());
			   BufferedReader br = new BufferedReader(isr);
			   while(br.ready()){
				   sb.append(br.readLine());
				   sb.append("\n");
			   }
			   br.close();
			   isr.close();
			   return sb.toString();
		   } catch (FileNotFoundException e) {
				e.printStackTrace();
		   } catch (IOException e) {
			   e.printStackTrace();
		   } catch (CoreException e) {
			e.printStackTrace();
		}
	   }
	   return "";
   }
   public String getNewFileContents(){
	   return (getComment() + "\n" + getContent()).replace("${nodeQID}", getQid());
   }
}