/*
 * Copyright 2009-2012 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.reqcoverage.eclipse.wizards;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.dialogs.WizardNewFileCreationPage;
import org.eclipse.ui.ide.IDE;

import com.unitesk.reqcoverage.Activator;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.wizards.ITestGenWizard;
import com.unitesk.requality.eclipse.wizards.TreeNodeSelectionPage;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tests.ProjectDialogSettingsHandler;

public class NewTestWizard extends Wizard implements ITestGenWizard {
	private DialogSettings settings = new DialogSettings("NewTestStepWizard");
	private ProjectDialogSettingsHandler settsLoader = new ProjectDialogSettingsHandler("NewTestStepWizard");
	public class TTCNNewFileCreationPage extends WizardNewFileCreationPage {
	    public TTCNNewFileCreationPage(IStructuredSelection selection) {
	        super("chooseTestFileName", selection);
	    }
	 
	    @Override
	    protected InputStream getInitialContents() {
	    	if(page == null)
	    		return null;
	        String text = page.getFileContents();
	        return new ByteArrayInputStream(text.getBytes());
	    }

		public void updateSettings() {
			setTitle( getTestWizard().getSettings().get("title") );
			setFileExtension( getTestWizard().getSettings().get("extension") );
		}

		private ITestGenWizard getTestWizard() {
			return (ITestGenWizard)getWizard();
		}
		private void setPath(String path){
			this.setContainerFullPath(Path.fromOSString(path));
		}

		private String getPath(){
			return getContainerFullPath().toOSString();
		}
	}
    private TTCNNewFileCreationPage createPage;
    private SelectTestTypePage selectpage;
    private NewTestPage page;
    private TreeNodeSelectionPage selnodepage;
    private ISelection selection;
    protected String targetqid;
	private TreeDB db;
	private Map<String, Map<String,String>> map;
    /**
     * Constructor for ReqWizard.
     */
    public NewTestWizard() {
        super();
        setNeedsProgressMonitor(true);
    }
    
    private String selectedId;
    
    public void setSelectedId(String val){
    	selectedId = val;
    	if(createPage!=null && selectedId != null){
    		createPage.updateSettings();
    		page.updateSettings();
    		selnodepage.updateSettings();
    	}
    }
    
    private String getTestGenName(){
    	Map<String,String> setts = getSettings();
    	String ret = "default";
    	if(setts!=null && setts.containsKey("name"))
    		ret = setts.get("name");
    	return ret.replace(" ", "_");
    }
    /**
     * Adding the page to the wizard.
     */
    public void addPages() {
    	if(selectedId == null){
    		selectpage = new SelectTestTypePage(new StructuredSelection(), getAvailableMap());
    		addPage(selectpage);
    	}
    	
    	createPage = new TTCNNewFileCreationPage(new StructuredSelection());
    	addPage(createPage);
        TreeDB db = getSelectedTreeDB();
        StructuredSelection sel;
        if(db != null)
        	sel = new StructuredSelection(db);
        else
        	sel = new StructuredSelection();
        selnodepage = new TreeNodeSelectionPage(sel);
        selnodepage.setNodeTypesFilters(new String[]{Requirement.TYPE_NAME, TestPurpose.TYPE_NAME});
        selnodepage.setChangeListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				if(event == null)
					setTargetQID(null, true);
				else
					if(!((StructuredSelection)event.getSelection()).isEmpty())
						setTargetQID(((TreeNode)((StructuredSelection)event.getSelection()).getFirstElement()).getUserFriendlyName(), true);
			}
		});
        
        addPage(selnodepage);
        page = new NewTestPage(selection);
        addPage(page);
        selnodepage.setTargetQIDs(targetqid);
        setSelectedId(selectedId);
        String savedPath = settings.get("path"+getTestGenName());
        if(savedPath!=null)
        	createPage.setPath(savedPath);

        String savedText = settings.get("text"+getTestGenName());
        if(savedText!=null)
        	page.setTSDescription(savedText);
    }

    public Map<String,Map<String,String>> getAvailableMap() {
    	if(map == null)
    		map = Activator.getDefault().getTestDescriptions();
		return map;
	}

	private TreeDB getSelectedTreeDB() {
    	if(!(selection instanceof StructuredSelection))
    		return db;
    	TreeDB db = this.db;
    	Object o = ((StructuredSelection)selection).getFirstElement();
    	if(o instanceof IProject)
    		db = TreesTracker.getTracker().getTree((IProject)o);
    	if(o instanceof TreeNode)
    		db = ((TreeNode)o).getTreeDB();
    	if(o instanceof TreeDB)
    		db = (TreeDB)o;
    	this.db = db;
    	return db;
	}

	@Override
    public boolean performFinish() {
		if(selectedId == null)
			return false;
        try {
        	settings.put("path"+getTestGenName(),createPage.getPath());
        	settings.put("text"+getTestGenName(),page.getRawContents());
        	settsLoader.saveSettings(settings);
        	IFile f = createPage.createNewFile();
    		if (f != null) {
    			IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), f);
    			return true;
    		}else
    			return false;
            //updatePerspective();
        } catch (WorkbenchException e) {
            e.printStackTrace();
        }
        return true;
    }

    /**
     * We will accept the selection in the workbench to see if we can initialize
     * from it.
     * 
     * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
     */
    public void init(IWorkbench workbench, IStructuredSelection selection) {
        this.selection = selection;
    }

	public String getPath() {
		return createPage.getContainerFullPath().toString();
	}
	
	@Override
	public String getTargetQID() {
		return targetqid;
	}
	
	public void setTargetQID(String targetqid, boolean b) {
		if(page!=null && !b)
			page.setTargetQID(targetqid, true);
		if(selnodepage!=null && !b)
			selnodepage.setTargetQIDs(targetqid);
		this.targetqid = targetqid;		
	}
	@Override
	public void setTreeDB(TreeDB treeDB) {
		this.db = treeDB;
		settsLoader.initOverProject(TreesTracker.getTracker().getProject(treeDB));
		settsLoader.loadSettings(settings);
	}
	
	@Override
	public Map<String, String> getSettings() {
		if(selectedId != null){
    		return getAvailableMap().get(selectedId);
    	}
		return null;
	}
}